USE [igops]
GO

DROP VIEW IF EXISTS business_role_v
DROP VIEW IF EXISTS suser_account_permission_v
DROP VIEW IF EXISTS current_user_access_v
DROP VIEW IF EXISTS suser_brole_v
DROP VIEW IF EXISTS deleted_reviewers_v
DROP VIEW IF EXISTS sgroup_indirect_suser_v
DROP VIEW IF EXISTS sgroup_path_v
DROP VIEW IF EXISTS sgroups_subgroups_v
DROP VIEW IF EXISTS suser_sod_case_info_v
DROP VIEW IF EXISTS sod_case_info_v
DROP VIEW IF EXISTS role_perm_policy_v
DROP VIEW IF EXISTS sod_policy_all_v
DROP VIEW IF EXISTS sod_policy_v
DROP VIEW IF EXISTS policy_owners_v
DROP VIEW IF EXISTS sod_case_violation_v
DROP VIEW IF EXISTS sod_policy_violation_v
DROP VIEW IF EXISTS spermission_associations_v
DROP VIEW IF EXISTS spermission_suser_v
DROP VIEW IF EXISTS business_role_nested_v
GO

CREATE VIEW business_role_v AS
  SELECT DISTINCT
    b_role.id AS business_role_id,
    b_role.name,
    b_role.description,
    b_role.grace_period,
    b_role.risk,
    b_role.update_time,
      (
      SELECT STUFF(
      (
        SELECT ' | ' + suser.display_name
        FROM
          suser
          INNER JOIN business_role_owner br_owner_1
            ON  suser.unique_user_id = br_owner_1.unique_owner_id
            AND suser.display_name IS NOT NULL
            AND suser.deleted = 0
            AND suser.effective = 1
        WHERE
          br_owner_1.business_role_id = b_role.id
        AND br_owner_1.is_user = 1
        GROUP BY suser.display_name
        FOR XML PATH('')
      ), 1, 3, '')
    ) AS user_owners,
    (
      SELECT STUFF(
      (
        SELECT ' | ' + sgroup.name
                FROM
          sgroup
          INNER JOIN business_role_owner br_owner_1
            ON  br_owner_1.unique_owner_id = sgroup.unique_group_id
            AND sgroup.name IS NOT NULL
            AND sgroup.deleted = 0
            AND sgroup.effective = 1
        WHERE
          br_owner_1.business_role_id = b_role.id
        AND br_owner_1.is_user = 0
        GROUP BY sgroup.name
        FOR XML PATH('')
      ), 1, 3, '')
    ) AS group_owners,
    b_role.approval_state,
    (
      SELECT DISTINCT suser.display_name
      FROM suser
      WHERE
        suser.unique_user_id = b_role.created_by
      AND suser.display_name IS NOT NULL
      AND suser.deleted = 0
      AND suser.effective = 1
    ) AS created_by,
    (
      SELECT DISTINCT suser.display_name
      FROM suser
      WHERE
        suser.unique_user_id = b_role.published_by
      AND suser.display_name IS NOT NULL
      AND suser.deleted = 0
      AND suser.effective = 1
    ) AS published_by,
    (
      SELECT DISTINCT suser.display_name
      FROM suser
      WHERE
        suser.unique_user_id = b_role.changed_by
      AND suser.display_name IS NOT NULL
      AND suser.deleted = 0
      AND suser.effective = 1
    ) AS changed_by,
    (
      SELECT DISTINCT suser.display_name
      FROM suser
      WHERE
        b_role.approved_by = suser.unique_user_id
      AND suser.display_name IS NOT NULL
      AND suser.deleted = 0
      AND suser.effective = 1
    ) AS approved_by,
    (
      SELECT STUFF(
      (
        SELECT ' | ' + suser.display_name
                FROM
          suser
          INNER JOIN auth_role_mapping ar_map
            ON  ar_map.unique_user_id = suser.unique_user_id
            AND suser.display_name IS NOT NULL
            AND suser.deleted = 0
            AND suser.effective = 1
          LEFT JOIN auth_scope
            ON  auth_scope.id = ar_map.auth_scope
        WHERE
          ar_map.unique_user_id IS NOT NULL
        AND ar_map.auth_role = 27
        AND auth_scope.unique_business_role_id = b_role.unique_id
        GROUP BY suser.display_name
        FOR XML PATH('')
      ), 1, 3, '')
    ) AS user_managers,
    (
      SELECT STUFF(
      (
        SELECT ' | ' + sgroup.name
                FROM
          sgroup
          INNER JOIN auth_role_mapping ar_map
            ON  ar_map.unique_group_id = sgroup.unique_group_id
            AND sgroup.name IS NOT NULL
          LEFT JOIN auth_scope
            ON  ar_map.auth_scope = auth_scope.id
        WHERE
          ar_map.unique_group_id IS NOT NULL
        AND ar_map.auth_role = 27
        AND auth_scope.unique_business_role_id = b_role.unique_id
        GROUP BY sgroup.name
        FOR XML PATH('')
      ), 1, 3, '')
    ) AS group_managers,
    (
      SELECT DISTINCT suser.display_name
            FROM
        suser
        INNER JOIN auth_role_mapping ar_map
          ON  ar_map.unique_user_id = suser.unique_user_id
          AND suser.display_name IS NOT NULL
          AND suser.deleted = 0 AND suser.effective = 1
        LEFT JOIN auth_scope
          ON  ar_map.auth_scope = auth_scope.id
      WHERE
        ar_map.auth_role = 29
      AND auth_scope.unique_business_role_id = b_role.unique_id
    ) AS user_fulfiller,
    (
      SELECT DISTINCT sgroup.name
            FROM
        sgroup
        INNER JOIN auth_role_mapping ar_map
          ON  ar_map.unique_group_id = sgroup.unique_group_id
          AND sgroup.name IS NOT NULL
          AND sgroup.deleted = 0
          AND sgroup.effective = 1
        LEFT JOIN auth_scope
          ON ar_map.auth_scope = auth_scope.id
      WHERE
        ar_map.auth_role = 29
      AND auth_scope.unique_business_role_id = b_role.unique_id
    ) AS group_fulfiller,
    b_role.approval_note,
        (
      CASE WHEN b_role.approval_policy IS NULL
      THEN
      (
        SELECT approval_policy.name FROM approval_policy
        WHERE approval_policy.global = 1 AND approval_policy.deleted = 0
      )
      ELSE ap.name
      END
    ) AS policy_name,
        (
      CASE WHEN b_role.approval_policy IS NULL
      THEN
      (
        SELECT approval_policy.implicit FROM approval_policy
        WHERE approval_policy.global = 1 AND approval_policy.deleted = 0
      )
            ELSE ap.implicit
      END
    ) AS approval_not_required,
        (
      CASE WHEN b_role.approval_policy IS NULL
      THEN
      (
        SELECT approval_policy.description FROM approval_policy
        WHERE approval_policy.global = 1 AND approval_policy.deleted = 0
      )
            ELSE ap.description
      END
    ) AS policy_description,
        (
      CASE WHEN b_role.approval_policy IS NULL
      THEN
      (
        SELECT approval_policy.creation_time FROM approval_policy
        WHERE approval_policy.global = 1 AND approval_policy.deleted = 0
      )
            ELSE ap.creation_time
      END
    ) AS policy_creation_time,
        (
      CASE WHEN b_role.approval_policy IS NULL
      THEN
      (
        SELECT approval_policy.update_time FROM approval_policy
        WHERE approval_policy.global = 1 AND approval_policy.deleted = 0
      )
            ELSE ap.update_time
      END
    ) AS last_policy_update_time,
    (
      SELECT STUFF (
      (
        SELECT ' | ' + suser.display_name
                FROM
          suser
          INNER JOIN approval_policy_memb apmemb
            ON  apmemb.unique_member_id = suser.unique_user_id
            AND suser.display_name IS NOT NULL
            AND suser.deleted = 0 AND suser.effective = 1
        WHERE
          apmemb.is_user = 1
        AND apmemb.approval_policy_id = b_role.approval_policy
        GROUP BY suser.display_name
        ORDER BY suser.display_name
        FOR XML PATH('')
      ), 1, 3, '')
    ) AS policy_users,
    (
      SELECT STUFF(
      (
        SELECT ' | ' + sgroup.name
                FROM
          sgroup
          INNER JOIN approval_policy_memb apmemb
            ON  apmemb.unique_member_id = sgroup.unique_group_id
            AND sgroup.name IS NOT NULL
            AND sgroup.deleted = 0 AND sgroup.effective = 1
        WHERE
          apmemb.is_user = 0
        AND apmemb.approval_policy_id = b_role.approval_policy
        GROUP BY sgroup.name
        ORDER BY sgroup.name
        FOR XML PATH('')
      ), 1, 3, '')
    ) AS policy_groups,
    br_crit.criteria AS membership_criteria,
    (
      SELECT STUFF(
      (
        SELECT ' | ' + suser.display_name
                FROM
          suser
          INNER JOIN business_role_incl_memb br_incl_mem
            ON  br_incl_mem.unique_member_id = suser.unique_user_id
            AND suser.display_name IS NOT NULL
            AND suser.deleted = 0
            AND suser.effective = 1
        WHERE
          br_incl_mem.is_user = 1
        AND br_incl_mem.business_role_id = b_role.id
        GROUP BY suser.display_name
        ORDER BY suser.display_name
        FOR XML PATH('')
            ), 1, 3, '')
    ) AS included_users,
    (
      SELECT STUFF(
      (
        SELECT ' | ' + sgroup.name
                FROM
          sgroup
          INNER JOIN business_role_incl_memb br_incl_mem
            ON  br_incl_mem.unique_member_id = sgroup.unique_group_id
            AND sgroup.name IS NOT NULL
            AND sgroup.deleted = 0 AND sgroup.effective = 1
        WHERE
          br_incl_mem.is_user = 0
        AND br_incl_mem.business_role_id = b_role.id
        GROUP BY sgroup.name
        ORDER BY sgroup.name
        FOR XML PATH('')
      ), 1, 3, '')
    ) AS included_groups,
    (
      SELECT STUFF(
      (
        SELECT ' | ' + suser.display_name
                FROM
          suser
          INNER JOIN business_role_excl_memb br_excl_mem
            ON  br_excl_mem.unique_member_id = suser.unique_user_id
            AND suser.display_name IS NOT NULL
            AND suser.deleted = 0 AND suser.effective = 1
        WHERE
          br_excl_mem.is_user = 1
        AND br_excl_mem.business_role_id = b_role.id
        GROUP BY suser.display_name
        ORDER BY suser.display_name
        FOR XML PATH('')
            ), 1, 3, '')
    ) AS excluded_users,
    (
      SELECT STUFF(
      (
        SELECT ' | ' + sgroup.name
                FROM
          sgroup
          INNER JOIN business_role_excl_memb br_excl_mem
            ON  br_excl_mem.unique_member_id = sgroup.unique_group_id
            AND sgroup.name IS NOT NULL
            AND sgroup.deleted = 0
            AND sgroup.effective = 1
        WHERE
          br_excl_mem.is_user = 0
        AND br_excl_mem.business_role_id = b_role.id
        GROUP BY sgroup.name
        ORDER BY sgroup.name
        FOR XML PATH('')
            ), 1, 3, '')
    ) AS excluded_groups,
    (
      SELECT STUFF(
      (
        SELECT ' | ' + spermission.name
                FROM
          spermission
          INNER JOIN business_role_incl_item br_incl_item
            ON  br_incl_item.unique_item_id = spermission.unique_permission_id
            AND spermission.name IS NOT NULL
            AND spermission.deleted = 0 AND spermission.effective = 1
        WHERE
          br_incl_item.item_type = 'PERMISSION'
        AND br_incl_item.business_role_id = b_role.id
        GROUP BY spermission.name
        ORDER BY spermission.name
        FOR XML PATH('')
            ), 1, 3, '')
    ) AS auth_permission,
    (
      SELECT STUFF(
      (
        SELECT ' | ' + role_policy.name
                FROM
          role_policy
          INNER JOIN business_role_incl_item br_incl_item
          ON  br_incl_item.unique_item_id = role_policy.unique_policy_id
          AND role_policy.deleted = 0
        WHERE
          br_incl_item.item_type = 'TECHNICAL_ROLE'
        AND br_incl_item.business_role_id = b_role.id
        GROUP BY role_policy.name
        ORDER BY role_policy.name
        FOR XML PATH('')
            ), 1, 3, '')
    ) AS auth_tech_role,
    (
      SELECT STUFF(
      (
        SELECT ' | ' + data_source.name
                FROM
          data_source
          INNER JOIN business_role_incl_item br_incl_item
            ON  br_incl_item.unique_item_id = data_source.id
            AND data_source.deleted = 0
        WHERE
          br_incl_item.item_type = 'APPLICATION'
        AND br_incl_item.business_role_id = b_role.id
        GROUP BY data_source.name
        ORDER BY data_source.name
        FOR XML PATH('')
      ), 1, 3, '')
    ) AS auth_application,

    (
      SELECT STUFF(
      (
        SELECT ' | ' + incl_brole2.name
                FROM
          business_role_incl_brole incl_brole2
          INNER JOIN business_role brole
            ON  brole.id = incl_brole2.business_role_id
        WHERE incl_brole2.business_role_id = b_role.id
        FOR XML PATH('')
      ), 1, 3, '') 
    ) AS included_broles

  FROM
    business_role b_role
    LEFT JOIN approval_policy ap ON b_role.approval_policy = ap.id
    LEFT JOIN business_role_owner br_owner ON b_role.id = br_owner.business_role_id
    LEFT JOIN business_role_memb_crit br_crit ON b_role.id = br_crit.business_role_id
  WHERE b_role.approval_state != 'ARCHIVE'
GO

CREATE VIEW suser_account_permission_v AS
    select
        case when sap.id is not null then
            convert(nvarchar(30),sau.id) + '.' + convert(nvarchar(30),sap.id)
        else
            convert(nvarchar(30),sau.id) + '.null'
        end as id,
        su.id as suser,
        su.collected_entity as collected_entity,
        sa.id as saccount,
        sp.id as spermission,
        sau.sversion as sversion,
        sau.id as saccount_user,
        sap.id as saccount_permission,
        sap.value as spermission_value,
        case when (sa.virtual is null or sa.virtual = 0) then
            sau.id
        else
            null
        end as real_saccount_user
    from saccount_user sau
    join saccount sa on sa.collected_entity = sau.saccount and sa.effective = 1 and sa.deleted = 0
    join suser su on su.collected_entity = sau.suser and su.effective = 1 and su.deleted = 0
    left join saccount_permission sap on sap.saccount = sau.saccount and sap.effective = 1
    left join spermission sp on sp.collected_entity = sap.spermission and sp.effective = 1 and sp.deleted = 0
    where
      sau.effective = 1
GO

CREATE VIEW business_role_nested_v AS
  WITH nested_broles(parent_id, child_id, depth, path) AS (
    SELECT
      base.parent_id,
      base.child_id,
      1,
      CAST(base.parent_id AS nvarchar(MAX))
    FROM ( SELECT 
        incl_brole.business_role_id AS parent_id,
        child.id AS child_id
      FROM business_role_incl_brole incl_brole
        JOIN business_role parent ON parent.id = incl_brole.business_role_id AND parent.approval_state = 'PUBLISHED'
        JOIN business_role child ON child.unique_id = incl_brole.unique_brole_id AND child.approval_state = 'PUBLISHED') base
        
    UNION ALL
    
    SELECT 
      base.parent_id,
      base.child_id,
      nb.depth + 1,
      nb.path + ', ' + CAST(base.parent_id AS nvarchar(MAX))
    FROM ( SELECT 
        incl_brole.business_role_id AS parent_id,
        child.id AS child_id
      FROM business_role_incl_brole incl_brole
        JOIN business_role parent ON parent.id = incl_brole.business_role_id AND parent.approval_state = 'PUBLISHED'
        JOIN business_role child ON child.unique_id = incl_brole.unique_brole_id AND child.approval_state = 'PUBLISHED') base
        JOIN nested_broles nb ON base.parent_id = nb.child_id
    )
  SELECT 
    nested_broles.parent_id,
    nested_broles.child_id,
    nested_broles.depth,
    nested_broles.path
  FROM nested_broles
GO

CREATE VIEW spermission_suser_v AS
SELECT
  sap.suser,
  perm.id permission_id,
  perm.name permission_name,
  perm.description permission_description,
  perm.type permission_type,
  data_source.name application_name,
  perm.risk permission_risk
FROM
  (SELECT id, name, description, type, risk, application FROM spermission WHERE (exclude_from_catalog = 0 OR exclude_from_catalog IS NULL) AND effective = 1) perm
  JOIN data_source ON perm.application = data_source.id
  JOIN (SELECT DISTINCT suser, spermission FROM suser_account_permission) sap ON perm.id = sap.spermission
GO

CREATE VIEW spermission_associations_v AS
  SELECT
    perm.id,
    perm.name AS permission,
    perm.type AS permission_type,
    perm.exclude_from_catalog,
    perm_assoc.name AS associated_permission,
    perm_assoc.type AS associated_permission_type,
    perm_entail.entailed AS associated_permission_id,
    perm_entail.distance
  FROM spermission_entailment perm_entail
    LEFT JOIN spermission perm ON perm.collected_entity = perm_entail.entailing
    LEFT JOIN spermission perm_assoc ON perm_assoc.collected_entity = perm_entail.entailed
  WHERE perm.deleted = 0 AND perm.effective = 1 AND perm_assoc.deleted = 0 AND perm_assoc.effective = 1
GO

CREATE VIEW sod_policy_violation_v AS
SELECT
  sod_case_info.id AS case_id,
  sod_case_info.state,
  sod_case_info.current_violation,
  sod_case_info.policy AS policy_id,
  sod_case_info.policy_name,
  sod_case_info.violator_name,
  sod_case_info.violator_title,
  sod_violation.suser,
  sod_violation.saccount,
  sod_condition.id AS condition_id,
  sod_condition.type AS cond_type,
  sod_condition.cond_order,
  sod_condition_item.unique_id,
  CASE
    WHEN sod_condition_item.type = 'PERMISSION' THEN spermission.name
    ELSE role_policy.name
  END AS item_name,
  sod_condition_item.type AS item_type,
  sod_condition_item.item_order
FROM sod_case_info
LEFT JOIN sod_condition ON sod_condition.policy = sod_case_info.policy
LEFT JOIN sod_condition_item ON sod_condition_item.condition = sod_condition.id
LEFT JOIN sod_violation ON sod_violation.id = sod_case_info.current_violation
LEFT JOIN spermission ON spermission.unique_permission_id = sod_condition_item.unique_id AND spermission.effective = 1 AND spermission.deleted = 0
LEFT JOIN role_policy ON role_policy.unique_policy_id = sod_condition_item.unique_id AND role_policy.deleted = 0
WHERE
  sod_condition_item.unique_id IN
    (SELECT DISTINCT spermission_1.unique_permission_id
      FROM saccount_user
        JOIN saccount_permission ON saccount_permission.saccount = saccount_user.saccount
        JOIN spermission spermission_1 ON spermission_1.id = saccount_permission.spermission
      WHERE saccount_user.saccount = sod_violation.saccount OR saccount_user.suser = sod_violation.suser)
  OR sod_condition_item.unique_id IN
    (SELECT DISTINCT role_policy_1.unique_policy_id
      FROM suser
        JOIN role_user ON role_user.suser = suser.id
        JOIN role_policy role_policy_1 ON role_policy_1.id = role_user.policy
      WHERE suser.id = sod_violation.suser)
GO

CREATE VIEW sod_case_violation_v AS
SELECT DISTINCT users.id,
            users.unique_user_id,
            users.display_name,
      policy.perm_id,
            policy.unique_id,
            policy.item_name,
            policy.policy_name
FROM suser users
LEFT JOIN (SELECT DISTINCT 
          spv1.UNIQUE_ID,
          spv1.item_name,
          spv1.perm_id,
          users1.unique_user_id, 
  
          (STUFF((SELECT ', ' + (CONCAT(spv.policy_name, CONCAT(': SOD-', RIGHT(CONVERT(varchar, CONVERT(varbinary(4), spv.case_id),2), 4 ))))
            FROM suser users
              LEFT JOIN (SELECT sc.id AS case_id,
                CASE
                  WHEN sv.suser IS NOT NULL THEN su.display_name
                  ELSE sa.name
                END AS violator_name,
                sp.name as policy_name,
                sod_condition_item.unique_id
                
               FROM sod_case sc
               LEFT JOIN sod_policy sp ON sc.policy = sp.id
               LEFT JOIN sod_violation sv ON sc.current_violation = sv.id
               LEFT JOIN suser su ON sv.suser = su.id
               LEFT JOIN saccount sa ON sv.saccount = sa.id
               LEFT JOIN sod_condition ON sod_condition.policy = sc.policy
               LEFT JOIN sod_condition_item ON sod_condition_item.condition = sod_condition.id
               LEFT JOIN spermission spermission_1 ON spermission_1.unique_permission_id = sod_condition_item.unique_id AND spermission_1.effective = 1 AND spermission_1.deleted = 0
               LEFT JOIN role_policy ON role_policy.unique_policy_id = sod_condition_item.unique_id AND role_policy.deleted = 0
               ) spv ON spv.violator_name = users.display_name

            WHERE users.unique_user_id = users1.unique_user_id AND spv.unique_id = spv1.unique_id AND spv.policy_name IS NOT NULL AND users.effective = 1 FOR XML PATH('')),1,1,'')) policy_name


            FROM suser users1
            LEFT JOIN (SELECT sc.id AS case_id,
                    spermission_1.id as perm_id,
                    CASE
                      WHEN sv.suser IS NOT NULL THEN su.display_name
                      ELSE sa.name
                    END AS violator_name,
                    sp.name as policy_name,
                    CASE
                    WHEN sod_condition_item.type = 'PERMISSION' THEN spermission_1.name
                    ELSE role_policy.name
                  END AS item_name,
                    sod_condition_item.unique_id
                
                   FROM sod_case sc
                   LEFT JOIN sod_policy sp ON sc.policy = sp.id
                   LEFT JOIN sod_violation sv ON sc.current_violation = sv.id
                   LEFT JOIN suser su ON sv.suser = su.id
                   LEFT JOIN saccount sa ON sv.saccount = sa.id
                   LEFT JOIN sod_condition ON sod_condition.policy = sc.policy
                   LEFT JOIN sod_condition_item ON sod_condition_item.condition = sod_condition.id
                   LEFT JOIN spermission spermission_1 ON spermission_1.unique_permission_id = sod_condition_item.unique_id AND spermission_1.effective = 1 AND spermission_1.deleted = 0
               LEFT JOIN role_policy ON role_policy.unique_policy_id = sod_condition_item.unique_id AND role_policy.deleted = 0) spv1 ON spv1.violator_name = users1.display_name

             WHERE spv1.policy_name IS NOT NULL AND users1.effective = 1
             GROUP BY users1.unique_user_id, spv1.perm_id, users1.effective, spv1.unique_id, spv1.item_name, spv1.UNIQUE_ID) policy ON policy.unique_user_id = users.unique_user_id
       
        WHERE policy.policy_name IS NOT NULL AND users.effective = 1
  GROUP BY users.id, users.unique_user_id, users.display_name, policy.unique_id, policy.item_name, policy.policy_name, policy.perm_id
GO

CREATE VIEW policy_owners_v AS
  SELECT
    policy_owner.is_user,
    policy_owner.unique_owner_id,
    policy_owner.role_policy,
    policy_owner.sod_policy,
    CASE
      WHEN policy_owner.is_user = 1 THEN suser.display_name
      WHEN policy_owner.is_user = 0 THEN sgroup.name
      ELSE NULL
    END AS owner_name
  FROM policy_owner
    LEFT JOIN suser ON suser.unique_user_id = policy_owner.unique_owner_id AND suser.effective = 1
    LEFT JOIN sgroup ON sgroup.unique_group_id = policy_owner.unique_owner_id AND sgroup.effective = 1
GO

CREATE VIEW sod_policy_v AS
SELECT
    sp.id AS policy_id,
    sp.deleted AS policy_status,
    change.display_name AS changed_by,
    creater.display_name AS created_by,
    sp.creation_time,
    sp.description AS policy_description,
    sp.detection_status,
    sp.name AS policy_name,
    sp.risk,
    sp.state,
    sp.unique_policy_id,
    sp.update_time,
    sp.control_period,
    sp.resolve,
    policy_owners.owners
  FROM sod_policy sp
  LEFT JOIN suser change ON change.unique_user_id = sp.changed_by AND change.deleted = 0 AND change.effective = 1
  LEFT JOIN suser creater ON creater.unique_user_id = sp.created_by AND creater.deleted = 0 AND creater.effective = 1
  LEFT JOIN (
  SELECT
    A.sod_policy,
    MAX(C.owner_name) AS owners
  FROM policy_owners_v A
    CROSS APPLY (
      SELECT STUFF((SELECT ' | ' + B.owner_name
      FROM policy_owners_v B
      WHERE B.sod_policy = A.sod_policy FOR XML PATH('')), 1, 3, ' ') as owner_name) C
  GROUP BY A.sod_policy) policy_owners ON policy_owners.sod_policy = sp.id
  WHERE sp.deleted = 0
GO

CREATE VIEW sod_policy_all_v AS
  SELECT
    sp.id AS policy_id,
    sp.deleted AS policy_status,
    change.display_name AS changed_by,
    creater.display_name AS created_by,
    sp.creation_time,
    sp.description AS policy_description,
    sp.detection_status,
    sp.name AS policy_name,
    sp.risk,
    sp.state,
    sp.unique_policy_id,
    sp.update_time,
    sp.control_period,
    sp.resolve,
    policy_owners.owners
  FROM sod_policy sp
    LEFT JOIN suser change ON change.unique_user_id = sp.changed_by AND change.deleted = 0 AND change.effective = 1
    LEFT JOIN suser creater ON creater.unique_user_id = sp.created_by AND creater.deleted = 0 AND creater.effective = 1
    LEFT JOIN (SELECT
        A.sod_policy,
        MAX(C.owner_name) AS owners
      FROM policy_owners_v A
      CROSS APPLY (SELECT 
        STUFF((SELECT ' | ' + B.owner_name
          FROM policy_owners_v B
          WHERE B.sod_policy = A.sod_policy FOR XML PATH('')), 1, 3, ' ') as owner_name) C
      GROUP BY A.sod_policy) policy_owners ON policy_owners.sod_policy = sp.id
GO

CREATE VIEW role_perm_policy_v AS
SELECT DISTINCT
  policy.id,
  policy.name AS policy_name,
  policy.unique_policy_id,
  policy.description AS policy_description,
  policy.created_by,
  policy.changed_by,
  policy.creation_time,
  policy.update_time,
  policy.deleted,
  CASE
    WHEN policy.state = 'INACTIVE' AND ISNULL(policy.detection_status,'INACTIVE') = 'INACTIVE' THEN 'Inactive'
    WHEN policy.state = 'ACTIVE' AND policy.detection_status IN ('COMPLETED','RUNNING') THEN 'Active'
    WHEN policy.state = 'INACTIVE' AND policy.detection_status = 'INVALID' THEN 'Invalid from inactive'
    WHEN policy.state = 'INVALID' AND policy.detection_status = 'INVALID' THEN 'Invalid from active'
    ELSE NULL
  END AS current_state,
  policy.state AS actual_state,
  policy.detection_status,
  policy.risk_mode AS policy_risk_mode,
  policy.risk AS policy_risk,
  policy.cost AS policy_cost,
  (SELECT STUFF((SELECT ' | ' + powner.unique_owner_id
    FROM policy_owners_v powner
    LEFT JOIN role_policy rperm ON rperm.id = powner.role_policy
    WHERE powner.is_user = 1 AND rperm.id = policy.id
    GROUP BY powner.unique_owner_id
    FOR XML PATH('')), 1, 3, '') AS user_owner_id) AS user_owner_id,
  (SELECT STUFF((SELECT ' | ' + powner.unique_owner_id
    FROM policy_owners_v powner
    LEFT JOIN role_policy rperm ON rperm.id = powner.role_policy
    WHERE powner.is_user = 0 AND rperm.id = policy.id
    GROUP BY powner.unique_owner_id
    FOR XML PATH('')), 1, 3, '') AS group_owner_id) AS group_owner_id,
  (SELECT STUFF((SELECT ' | ' + powner.owner_name
    FROM policy_owners_v powner
    LEFT JOIN role_policy rperm ON rperm.id = powner.role_policy
    WHERE powner.is_user = 1 AND rperm.id = policy.id
    GROUP BY powner.owner_name
    FOR XML PATH('')), 1, 3, '') AS user_owner_id) AS user_owners,
  (SELECT STUFF((SELECT ' | ' + powner.owner_name
    FROM policy_owners_v powner
    LEFT JOIN role_policy rperm ON rperm.id = powner.role_policy
    WHERE powner.is_user = 0 AND rperm.id = policy.id
    GROUP BY powner.owner_name
    FOR XML PATH('')), 1, 3, '') AS user_owner_id) AS group_owners,
  perm.unique_permission_id,
  perm.name AS perm_name,
  perm_user.unique_user_id AS permission_user_id,
  perm_user.display_name AS permission_user_name
FROM
  role_policy policy
  LEFT JOIN policy_owner ON policy.id = policy_owner.role_policy
  LEFT JOIN role_permission role_perm ON  policy.id = role_perm.policy
  LEFT JOIN spermission perm ON role_perm.unique_permission_id = perm.unique_permission_id AND perm.effective = 1
  LEFT JOIN suser_account_permission_role role_user ON  role_user.role = policy.id
  LEFT JOIN suser perm_user ON  perm_user.id = role_user.suser;
GO

CREATE VIEW sod_case_info_v AS
SELECT
  sc.id,
    (
    CASE
      WHEN sc.state = 'APPROVED'
        AND sc.approval_expire_time IS NOT NULL
        AND sc.approval_expire_time < GETDATE()
        THEN 'APPROVAL_EXPIRED'
      ELSE sc.state
    END
  ) AS state,
  sc.approval_expire_time,
  sc.current_violation,
  sc.policy,
  sv.suser AS violator_is_user,
    (
    CASE
      WHEN sv.suser IS NOT NULL THEN su.display_name
      ELSE sa.name
    END
  ) AS violator_name,
  (
      CASE
        WHEN sv.suser IS NOT NULL THEN su.unique_user_id
      ELSE sa.unique_account_id
    END
  ) AS violator_id,
    (
    CASE
      WHEN sv.suser IS NOT NULL THEN su.job_title
      ELSE NULL
    END
  ) AS violator_title,
  sp.name AS policy_name
FROM
  sod_case sc
  LEFT JOIN sod_policy sp ON sc.policy = sp.id
  LEFT JOIN sod_violation sv ON sc.current_violation = sv.id
  LEFT JOIN suser su ON sv.suser = su.id
  LEFT JOIN saccount sa ON sv.saccount = sa.id
GO

CREATE VIEW suser_sod_case_info_v AS
SELECT 
    suser.id,
    suser.collected_entity,
    suser.display_name,
    suser.job_title,
    suser_email.email,
    suser.department,
    suser.user_id,
    suser.first_name,
    suser.last_name,
    suser.risk,
    suser.unique_user_id,
    supervisor.display_name AS supervisor,
    sci.violations
FROM suser
     LEFT JOIN suser supervisor ON supervisor.unique_user_id = suser.supervisor_unique_id AND supervisor.effective = 1 AND supervisor.deleted = 0
     LEFT JOIN ( SELECT 
                     suser1.id AS suser_id,
                     suser1.display_name,
                     STUFF((SELECT ' | ' + CAST(suser_email1.email AS nvarchar(MAX)) FROM suser suser2
                     JOIN suser_email suser_email1 ON suser2.id = suser_email1.suser AND suser2.effective = 1 AND suser2.deleted = 0
                     WHERE suser2.id = suser1.id FOR XML PATH('')),1,3,'') AS email
                 FROM suser suser1
                     JOIN suser_email suser_email1 ON suser1.id = suser_email1.suser AND suser1.effective = 1 AND suser1.deleted = 0
                 GROUP BY suser1.id, suser1.display_name) suser_email ON suser.id = suser_email.suser_id
     LEFT JOIN ( SELECT 
                     sci1.violator_name,
                     sci1.violator_id,
                     STUFF((SELECT ', ' + CAST(CONCAT(sci2.policy_name,(CONCAT(': SOD-', RIGHT(CONVERT(varchar, CONVERT(varbinary(4), sci2.id), 2), 4)))) AS nvarchar(MAX))
                     FROM sod_case_info_v sci2
                     WHERE sci2.violator_id = sci1.violator_id FOR XML PATH('')),1,2,'') AS violations
                 FROM sod_case_info_v sci1
                 GROUP BY sci1.violator_name, sci1.violator_id) sci ON sci.violator_id = suser.unique_user_id
WHERE suser.effective = 1 AND suser.deleted = 0
GO

CREATE VIEW sgroups_subgroups_v AS
  SELECT
    sub.supergroups,
    parent.name AS parent_name,
    sub.subgroups,
    child.name AS child_name
  FROM sgroup_subgroups sub
    LEFT JOIN sgroup parent ON parent.id = sub.supergroups
    LEFT JOIN sgroup child ON child.id = sub.subgroups
  WHERE parent.effective = 1 AND parent.deleted = 0 AND child.effective = 1 AND child.deleted = 0
GO

CREATE VIEW sgroup_path_v AS
WITH search_graph(
    id, parent_name, subgroups, child_name, depth, path, group_path, cycle)
  AS
  (
        SELECT
      g.supergroups, 
      g.parent_name, 
      g.subgroups, 
      g.child_name, 
      1,
      CONVERT(nvarchar(MAX), g.supergroups),
            CONVERT(nvarchar(MAX),g.parent_name) + CONVERT(nvarchar(MAX),'\'),
      0
        FROM
      sgroups_subgroups_v g
        UNION ALL
        SELECT
      g.supergroups,   
        g.parent_name,
        g.subgroups,
        g.child_name,
        sg.depth+1,
      path + ', ' + CONVERT(nvarchar(MAX), g.supergroups),
      CONVERT(nvarchar(MAX), sg.group_path) + CONVERT(nvarchar(MAX), g.parent_name) + CONVERT(nvarchar(MAX),'\'),
      CASE WHEN sg.path like (('%' + CAST(g.supergroups as nvarchar(max))) + '%') THEN 1 ELSE 0 END
        FROM 
      sgroups_subgroups_v g, search_graph sg
    WHERE 
      g.supergroups = sg.subgroups AND sg.cycle = 0
    )

  SELECT
    search_graph.id,
    search_graph.parent_name,
    search_graph.subgroups,
    search_graph.child_name,
    search_graph.depth,
    search_graph.path,
    search_graph.group_path,
    search_graph.cycle
  FROM search_graph
GO

CREATE VIEW sgroup_indirect_suser_v AS
SELECT
  groups.parent_id,
  parent.name parent_name,
  groups.subgroups child_id,
  child.name child_name,
  suser.id,
  suser.unique_user_id,
  suser.display_name,
  suser_email.email,
  suser.department,
  suser.job_title
FROM
  (SELECT
    A.subgroups,  
    Split.a.value('.', 'bigint') AS parent_id  
  FROM (SELECT 
   subgroups,  
   CAST ('<M>' + REPLACE(path, ', ', '</M><M>') + '</M>' AS XML) AS Data  
   FROM sgroup_path_v) AS A CROSS APPLY Data.nodes ('/M') AS Split(a)
  UNION
  SELECT  
    subgroups, 
    id 
  FROM sgroup_path_v) groups
  JOIN sgroup parent ON groups.parent_id = parent.id AND parent.effective = '1' AND parent.deleted = '0'
  JOIN sgroup child ON groups.subgroups = child.id AND child.effective = '1' AND child.deleted = '0'
  JOIN ( SELECT
    sgroup_user.suser AS suser_id,
    sgroup.id
  FROM sgroup_user
    JOIN sgroup ON sgroup_user.sgroup = sgroup.id AND sgroup.deleted = '0' AND sgroup.effective = '1') sgroup_suser ON groups.subgroups = sgroup_suser.id
  JOIN suser ON sgroup_suser.suser_id = suser.collected_entity AND suser.effective = '1'
  LEFT JOIN (SELECT DISTINCT 
      suser.id suser,
      suser.display_name,
      STUFF((SELECT DISTINCT ' | ' + suser_email.email FROM suser suser1
      JOIN suser_email ON suser.id = suser_email.suser AND suser1.effective = 1 AND suser1.deleted = 0
      WHERE suser1.id = suser.id
      FOR XML PATH('')),1,3,'') email
    FROM suser
    JOIN suser_email ON suser.id = suser_email.suser AND suser.effective = 1 AND suser.deleted = 0
    GROUP BY suser.id, suser.display_name) suser_email ON suser.id = suser_email.suser
GO

CREATE VIEW deleted_reviewers_v AS
  WITH all_reviewers AS (
      SELECT 
        item.id item_id,
        single_reviewer.display_name reviewer,
        single_reviewer.unique_user_id,
        single_reviewer.deleted,
        single_reviewer.effective,
        single_reviewer.timestamp time_deletion,
        defn.name review_name,
        instance.id review_instance_id,
        instance.status,
        item.item_type,
        item_user.display_name AS user_display_name,
        CASE
          WHEN item.item_type = 'PERMISSION' THEN perm.name
          WHEN item.item_type = 'ACCOUNT' THEN acct.name
          WHEN item.item_type = 'ROLE' THEN role.name
          WHEN item.item_type = 'BUSINESS_ROLE' THEN brole.name
        END review_item
      FROM review_item item
        JOIN (SELECT 
            action.review_item_id item_id,
            suser.display_name,
            suser.unique_user_id,
            suser.deleted,
            suser.effective,
            COALESCE(suser.last_modified, (SELECT min(timestamp) FROM sversion JOIN data_producer dp ON sversion.data_producer = dp.id WHERE dp.data_producer_type = 'UNIFIER' AND sversion.id > suser.sversion)) timestamp
          FROM review_item_action action
            JOIN reviewer_instance ri ON action.reviewer_instance_id = ri.id AND ri.shared_queue = 0
            JOIN suser ON ri.single_reviewer_unique_id = suser.unique_user_id AND suser.effective = 1
          WHERE action.review_item_status <> 'OVERRIDDEN' AND action.action <> 'REASSIGNED') single_reviewer ON item.id = single_reviewer.item_id
        JOIN suser item_user ON item_user.unique_user_id = item.unique_user_id AND item_user.effective = 1
        JOIN review_instance instance ON item.review_instance_id = instance.id AND instance.status IN ('RUNNING','PREVIEW')
        JOIN (SELECT
            review_instance.id inst_id,
            CASE WHEN staged_defn.name IS NULL THEN defn.name ELSE staged_defn.name END AS name
          FROM review_definition defn
            JOIN review_instance ON defn.id = review_instance.review_definition
            LEFT JOIN review_definition staged_defn ON staged_defn.id = review_instance.staged_review_definition_id) defn ON instance.id = defn.inst_id
        LEFT JOIN spermission perm ON perm.unique_permission_id = item.unique_permission_id AND perm.effective = 1
        LEFT JOIN saccount acct ON acct.unique_account_id = item.unique_account_id AND acct.effective = 1
        LEFT JOIN business_role brole ON brole.id = item.business_role_id
        LEFT JOIN role_policy role ON role.id = item.role_id
    UNION
      SELECT 
        item.id AS item_id,
        multi_reviewer.display_name AS reviewer,
        multi_reviewer.unique_user_id,
        multi_reviewer.deleted,
        multi_reviewer.effective,
        multi_reviewer.timestamp AS time_deletion,
        defn.name AS review_name,
        instance.id AS review_instance_id,
        instance.status,
        item.item_type,
        item_user.display_name AS user_display_name,
        CASE
          WHEN item.item_type = 'PERMISSION' THEN perm.name
          WHEN item.item_type = 'ACCOUNT' THEN acct.name
          WHEN item.item_type = 'ROLE' THEN role.name
          WHEN item.item_type = 'BUSINESS_ROLE' THEN brole.name
        END AS review_item
      FROM review_item item
        JOIN (SELECT 
              action.review_item_id AS item_id,
              action.review_instance_id,
              suser.display_name,
              suser.unique_user_id,
              suser.deleted,
              suser.effective,
              COALESCE(suser.last_modified, (SELECT min(timestamp) FROM sversion JOIN data_producer dp ON sversion.data_producer = dp.id WHERE dp.data_producer_type = 'UNIFIER' AND sversion.id > suser.sversion)) timestamp
            FROM review_item_action action
              JOIN reviewer_instance ri ON action.reviewer_instance_id = ri.id AND ri.shared_queue = 1
              JOIN reviewer_set_instance rsi ON ri.id = rsi.reviewer_instance_id
              JOIN (SELECT 
                  suser.display_name,
                  suser.unique_user_id,
                  suser.deleted,
                  suser.effective,
                  suser.sversion,
                  suser.last_modified
                FROM suser
                WHERE suser.effective = 1) suser ON rsi.unique_user_id = suser.unique_user_id
            WHERE action.action <> 'REASSIGNED'
          UNION
            SELECT 
              action.review_item_id AS item_id,
              action.review_instance_id,
              sgroup_users.display_name,
              sgroup_users.unique_user_id,
              sgroup_users.deleted,
              sgroup_users.effective,
              COALESCE(sgroup_users.last_modified, (SELECT min(timestamp) FROM sversion JOIN data_producer dp ON sversion.data_producer = dp.id WHERE dp.data_producer_type = 'UNIFIER' AND sversion.id > sgroup_users.sversion)) timestamp
            FROM review_item_action action
              JOIN reviewer_instance ri ON action.reviewer_instance_id = ri.id AND ri.shared_queue = 1
              JOIN reviewer_set_instance rsi ON ri.id = rsi.reviewer_instance_id
              JOIN (SELECT 
                    sgroup.unique_group_id,
                    suser.display_name,
                    suser.unique_user_id,
                    suser.deleted,
                    suser.effective,
                    suser.sversion,
                    suser.last_modified
                  FROM sgroup
                    JOIN sgroup_user gu ON gu.sgroup = sgroup.id
                    JOIN suser ON gu.suser = suser.collected_entity AND suser.effective = 1
                UNION
                  SELECT 
                    sgroup.unique_group_id,
                    suser.display_name,
                    suser.unique_user_id,
                    suser.deleted,
                    suser.effective,
                    suser.sversion,
                    suser.last_modified
                  FROM sgroup
                    JOIN sgroup_user gu ON gu.sgroup = sgroup.id
                    JOIN suser ON gu.suser = suser.collected_entity AND (SELECT MAX(sversion) FROM sgroup_user WHERE sgroup = sgroup.id) = suser.sversion AND sgroup.effective = 1
                UNION 
                  SELECT 
                    sgroup.unique_group_id,
                    suser.display_name,
                    suser.unique_user_id,
                    suser.deleted,
                    suser.effective,
                    suser.sversion,
                    suser.last_modified
                  FROM sgroup
                    JOIN sgroup_indirect_suser_v indirect ON indirect.parent_id = sgroup.id
                    JOIN suser ON indirect.unique_user_id = suser.unique_user_id AND suser.effective = 1) sgroup_users ON rsi.unique_group_id = sgroup_users.unique_group_id
            WHERE action.action <> 'REASSIGNED') multi_reviewer ON item.id = multi_reviewer.item_id
        JOIN suser item_user ON item_user.unique_user_id = item.unique_user_id AND item_user.effective = 1
        JOIN review_instance instance ON item.review_instance_id = instance.id AND instance.status IN ('RUNNING','PREVIEW')
        JOIN (SELECT
            review_instance.id inst_id,
            CASE WHEN staged_defn.name IS NULL THEN defn.name ELSE staged_defn.name END AS name
          FROM review_definition defn
            JOIN review_instance ON defn.id = review_instance.review_definition
            LEFT JOIN review_definition staged_defn ON staged_defn.id = review_instance.staged_review_definition_id) defn ON instance.id = defn.inst_id
        LEFT JOIN spermission perm ON perm.unique_permission_id = item.unique_permission_id AND perm.effective = 1
        LEFT JOIN saccount acct ON acct.unique_account_id = item.unique_account_id AND acct.effective = 1
        LEFT JOIN business_role brole ON brole.id = item.business_role_id
        LEFT JOIN role_policy role ON role.id = item.role_id
  )
  SELECT 
    deleted_reviewers.item_id,
    deleted_reviewers.reviewer AS deleted_reviewer,
    deleted_reviewers.unique_user_id,
    curr_reviewers.display_name AS curr_reviewers,
    deleted_reviewers.time_deletion,
    deleted_reviewers.review_name,
    deleted_reviewers.review_instance_id,
    deleted_reviewers.status,
    deleted_reviewers.item_type,
    deleted_reviewers.review_item,
    deleted_reviewers.user_display_name
  FROM all_reviewers deleted_reviewers
    LEFT JOIN (SELECT 
        ar.item_id,
        STUFF((SELECT ' | ' + CAST(ar1.reviewer AS nvarchar(MAX)) FROM all_reviewers ar1
          WHERE ar.item_id = ar1.item_id AND ar1.deleted = 0 AND ar1.effective = 1
          GROUP BY ar1.item_id, ar1.reviewer
          ORDER BY ar1.reviewer FOR XML PATH('')), 1, 3, '') display_name
      FROM all_reviewers ar
      WHERE ar.deleted = 0 AND ar.effective = 1
      GROUP BY ar.item_id) curr_reviewers ON curr_reviewers.item_id = deleted_reviewers.item_id
  WHERE deleted_reviewers.deleted = 1 AND deleted_reviewers.effective = 1;
GO

CREATE VIEW suser_brole_v AS 
  SELECT 
    brole.id brole_id,
    brole.name brole_name,
    suser.unique_user_id user_id,
    suser.display_name user_name,
    brole.description,
    brole.risk,
    brole.update_time
  FROM business_role brole
    JOIN (SELECT
      memb.business_role_id,
      COALESCE(suser.unique_user_id, direct.unique_user_id, indirect.unique_user_id) user_id
    FROM business_role_incl_memb memb
      LEFT JOIN suser ON suser.unique_user_id = memb.unique_member_id AND suser.effective = 1 AND suser.deleted = 0
      LEFT JOIN sgroup ON sgroup.unique_group_id = memb.unique_member_id AND sgroup.effective = 1
      LEFT JOIN (SELECT 
          sgroup.id,
          sgroup.unique_group_id,
          suser.unique_user_id,
          suser.display_name
        FROM sgroup 
          JOIN sgroup_user ON sgroup.id = sgroup_user.sgroup
          JOIN suser ON sgroup_user.suser = suser.collected_entity AND suser.effective = 1 AND suser.deleted = 0
        WHERE sgroup.effective = 1) direct ON sgroup.unique_group_id = direct.unique_group_id
      LEFT JOIN sgroup_indirect_suser_v indirect ON sgroup.id = indirect.parent_id
    WHERE NOT (memb.is_user = 0 AND direct.unique_group_id IS NULL AND indirect.parent_id IS NULL)
    
    EXCEPT
    
    SELECT
      excl_memb.business_role_id,
      COALESCE(suser.unique_user_id, direct.unique_user_id, indirect.unique_user_id) user_id
    FROM business_role_excl_memb excl_memb
    LEFT JOIN suser ON suser.unique_user_id = excl_memb.unique_member_id AND suser.effective = 1 AND suser.deleted = 0
    LEFT JOIN sgroup ON sgroup.unique_group_id = excl_memb.unique_member_id AND sgroup.effective = 1
    LEFT JOIN (SELECT 
          sgroup.id,
          sgroup.unique_group_id,
          suser.unique_user_id,
          suser.display_name
        FROM sgroup 
          JOIN sgroup_user ON sgroup.id = sgroup_user.sgroup
          JOIN suser ON sgroup_user.suser = suser.collected_entity AND suser.effective = 1 AND suser.deleted = 0
        WHERE sgroup.effective = 1) direct ON sgroup.unique_group_id = direct.unique_group_id
      LEFT JOIN sgroup_indirect_suser_v indirect ON sgroup.id = indirect.parent_id
    WHERE NOT (excl_memb.is_user = 0 AND direct.unique_group_id IS NULL AND indirect.parent_id IS NULL)) users ON users.business_role_id = brole.id
    JOIN suser ON users.user_id = suser.unique_user_id AND suser.effective = 1
  WHERE brole.approval_state = 'PUBLISHED'

  UNION 

  SELECT 
    indirect_brole_users.parent_id brole_id, 
    indirect_brole_users.parent_name brole_name, 
    indirect_brole_users.unique_user_id user_id, 
    indirect_brole_users.display_name user_name,
    indirect_brole_users.description,
    indirect_brole_users.risk,
    indirect_brole_users.update_time
  FROM
    (SELECT
      broles.parent_id,
      parent.name parent_name,
      broles.child_id,
      child.name,
      parent.description,
      parent.update_time,
      suser.unique_user_id,
      suser.display_name,
      parent.risk
    FROM (SELECT
      A.child_id,  
      Split.a.value('.', 'bigint') AS parent_id  
    FROM (SELECT 
     child_id,  
     CAST ('<M>' + REPLACE(path, ', ', '</M><M>') + '</M>' AS XML) AS Data  
     FROM business_role_nested_v) AS A CROSS APPLY Data.nodes ('/M') AS Split(a)
    UNION
    SELECT  
      child_id, 
      parent_id 
    FROM business_role_nested_v) broles
        JOIN business_role parent ON broles.parent_id = parent.id
        JOIN business_role child ON broles.child_id = child.id
        JOIN (SELECT
            memb.business_role_id,
            COALESCE(suser.unique_user_id, direct.unique_user_id, indirect.unique_user_id) user_id
          FROM business_role_incl_memb memb
        LEFT JOIN suser ON suser.unique_user_id = memb.unique_member_id AND suser.effective = 1 AND suser.deleted = 0
        LEFT JOIN sgroup ON sgroup.unique_group_id = memb.unique_member_id AND sgroup.effective = 1
        LEFT JOIN (SELECT 
            sgroup.id,
            sgroup.unique_group_id,
            suser.unique_user_id,
            suser.display_name
          FROM sgroup 
            JOIN sgroup_user ON sgroup.id = sgroup_user.sgroup
            JOIN suser ON sgroup_user.suser = suser.collected_entity AND suser.effective = 1 AND suser.deleted = 0
          WHERE sgroup.effective = 1) direct ON sgroup.unique_group_id = direct.unique_group_id
        LEFT JOIN sgroup_indirect_suser_v indirect ON sgroup.id = indirect.parent_id
      WHERE NOT (memb.is_user = 0 AND direct.unique_group_id IS NULL AND indirect.parent_id IS NULL)

      EXCEPT

      SELECT
        excl_memb.business_role_id,
        COALESCE(suser.unique_user_id, direct.unique_user_id, indirect.unique_user_id) user_id
      FROM business_role_excl_memb excl_memb
        LEFT JOIN suser ON suser.unique_user_id = excl_memb.unique_member_id AND suser.effective = 1 AND suser.deleted = 0
        LEFT JOIN sgroup ON sgroup.unique_group_id = excl_memb.unique_member_id AND sgroup.effective = 1
        LEFT JOIN (SELECT 
            sgroup.id,
            sgroup.unique_group_id,
            suser.unique_user_id,
            suser.display_name
          FROM sgroup 
            JOIN sgroup_user ON sgroup.id = sgroup_user.sgroup
            JOIN suser ON sgroup_user.suser = suser.collected_entity AND suser.effective = 1 AND suser.deleted = 0
          WHERE sgroup.effective = 1) direct ON sgroup.unique_group_id = direct.unique_group_id
        LEFT JOIN sgroup_indirect_suser_v indirect ON sgroup.id = indirect.parent_id
      WHERE NOT (excl_memb.is_user = 0 AND direct.unique_group_id IS NULL AND indirect.parent_id IS NULL)) groups_users ON child.id = groups_users.business_role_id
    JOIN suser ON groups_users.user_id = suser.unique_user_id AND suser.effective = 1 AND suser.deleted = 0) indirect_brole_users
GO

CREATE VIEW current_user_access_v AS
  WITH current_user_access AS (
    SELECT DISTINCT 
      users.id,
      users.display_name,
      users.job_title,
      users.email,
      users.department,
      users.user_id,
      users.first_name,
      users.last_name,
      users.risk,
      users.unique_user_id,
      users.supervisor,
      users.violations,
        CASE
          WHEN sgroup.group_id IS NOT NULL THEN '1-GROUP'
          ELSE '1-NO-GROUP'
        END detail_type,
      sgroup.group_id  group_id,
      sgroup.name  group_name,
      sgroup.short_description  group_description,
      NULL permission_id,
      NULL permission_name,
      NULL permission_description,
      NULL permission_type,
      NULL permission_application_name,
      NULL permission_risk,
      NULL permission_root,
      NULL permission_policy_violated,
      NULL permission_entitlements,
      NULL brole_id,
      NULL brole_name,
      NULL brole_description,
      NULL brole_risk,
      NULL brole_update_time,
      NULL supervisee_id,
      NULL supervisee_display_name,
      NULL supervisee_email,
      NULL supervisee_department,
      NULL supervisee_job_title,
      NULL trole_policy_id,
      NULL trole_item_name,
      NULL trole_unique_policy_id,
      NULL trole_description,
      NULL trole_risk,
      NULL trole_policy_violated,
      NULL account_name,
      NULL account_id,
      NULL account_risk,
      NULL account_description,
      NULL account_application_name
    FROM suser_sod_case_info_v users
      LEFT JOIN sgroup_user direct ON direct.suser = users.collected_entity
      LEFT JOIN sgroup_indirect_suser_v indirect ON users.id = indirect.id AND direct.sgroup = indirect.child_id
      LEFT JOIN sgroup ON sgroup.id = direct.sgroup OR sgroup.id = indirect.parent_id
    UNION ALL
    SELECT 
      users.id,
      users.display_name,
      users.job_title,
      users.email,
      users.department,
      users.user_id,
      users.first_name,
      users.last_name,
      users.risk,
      users.unique_user_id,
      users.supervisor,
      users.violations,
        CASE
          WHEN perm.permission_id IS NOT NULL THEN '2-PERMISSION'
          ELSE '2-NO-PERM'
        END detail_type,
      NULL group_id,
      NULL group_name,
      NULL group_description,
      perm.permission_id,
      perm.permission_name,
      perm.permission_description,
      perm.permission_type,
      perm.application_name  permission_application_name,
      perm.permission_risk,
      root_perm.permission  permission_root,
      spv.policy_name  permission_policy_violated,
      STUFF((SELECT ' | ' + CAST(ent.associated_permission AS nvarchar(MAX)) FROM suser_sod_case_info_v users1
          LEFT JOIN spermission_suser_v perm ON users.id = perm.suser
          LEFT JOIN (SELECT DISTINCT ent.id, ent.associated_permission FROM spermission_associations_v ent) ent ON ent.id = perm.permission_id
          LEFT JOIN spermission perm1 ON perm1.id = perm.permission_id
          LEFT JOIN spermission_associations_v root_perm ON root_perm.associated_permission_id = perm1.collected_entity
          LEFT JOIN sod_policy_violation_v spv ON spv.unique_id = perm1.unique_permission_id
          WHERE users1.id = users.id
          ORDER BY ent.associated_permission FOR XML PATH('')), 1, 3, '') permission_entitlements,
      NULL brole_id,
      NULL brole_name,
      NULL brole_description,
      NULL brole_risk,
      NULL brole_update_time,
      NULL supervisee_id,
      NULL supervisee_display_name,
      NULL supervisee_email,
      NULL supervisee_department,
      NULL supervisee_job_title,
      NULL trole_policy_id,
      NULL trole_item_name,
      NULL trole_unique_policy_id,
      NULL trole_description,
      NULL trole_risk,
      NULL trole_policy_violated,
      NULL account_name,
      NULL account_id,
      NULL account_risk,
      NULL account_description,
      NULL account_application_name
    FROM suser_sod_case_info_v users
      LEFT JOIN spermission_suser_v perm ON users.id = perm.suser
      LEFT JOIN (SELECT DISTINCT ent.id, ent.associated_permission FROM spermission_associations_v ent) ent ON ent.id = perm.permission_id
      LEFT JOIN spermission perm1 ON perm1.id = perm.permission_id
      LEFT JOIN spermission_associations_v root_perm ON root_perm.associated_permission_id = perm1.collected_entity
      LEFT JOIN sod_policy_violation_v spv ON spv.unique_id = perm1.unique_permission_id
    GROUP BY users.id, users.display_name, users.job_title, users.email, users.department, users.user_id, users.first_name, users.last_name, users.risk, users.unique_user_id, users.supervisor, users.violations, perm.permission_id, perm.permission_name, perm.permission_description, perm.permission_type, perm.application_name, perm.permission_risk, root_perm.permission, spv.policy_name
    UNION ALL
    SELECT 
      users.id,
      users.display_name,
      users.job_title,
      users.email,
      users.department,
      users.user_id,
      users.first_name,
      users.last_name,
      users.risk,
      users.unique_user_id,
      users.supervisor,
      users.violations,
        CASE
          WHEN brole.brole_id IS NOT NULL THEN '3-BROLE'
          ELSE '3-NO-BROLE'
        END detail_type,
      NULL group_id,
      NULL group_name,
      NULL group_description,
      NULL permission_id,
      NULL permission_name,
      NULL permission_description,
      NULL permission_type,
      NULL permission_application_name,
      NULL permission_risk,
      NULL permission_root,
      NULL permission_policy_violated,
      NULL permission_entitlements,
      brole.brole_id,
      brole.brole_name,
      brole.description brole_description,
      brole.risk brole_risk,
      brole.update_time  brole_update_time,
      NULL supervisee_id,
      NULL supervisee_display_name,
      NULL supervisee_email,
      NULL supervisee_department,
      NULL supervisee_job_title,
      NULL trole_policy_id,
      NULL trole_item_name,
      NULL trole_unique_policy_id,
      NULL trole_description,
      NULL trole_risk,
      NULL trole_policy_violated,
      NULL account_name,
      NULL account_id,
      NULL account_risk,
      NULL account_description,
      NULL account_application_name
    FROM suser_brole_v brole
      RIGHT JOIN suser_sod_case_info_v users ON users.unique_user_id = brole.user_id
    UNION ALL
    SELECT 
      users.id,
      users.display_name,
      users.job_title,
      users.email,
      users.department,
      users.user_id,
      users.first_name,
      users.last_name,
      users.risk,
      users.unique_user_id,
      users.supervisor,
      users.violations,
        CASE
          WHEN supervisee.unique_user_id IS NOT NULL THEN '4-SUPERVISEE'
          ELSE '4-NO-SUPERVISEE'
        END detail_type,
      NULL group_id,
      NULL group_name,
      NULL group_description,
      NULL permission_id,
      NULL permission_name,
      NULL permission_description,
      NULL permission_type,
      NULL permission_application_name,
      NULL permission_risk,
      NULL permission_root,
      NULL permission_policy_violated,
      NULL permission_entitlements,
      NULL brole_id,
      NULL brole_name,
      NULL brole_description,
      NULL brole_risk,
      NULL brole_update_time,
      supervisee.unique_user_id supervisee_id,
      supervisee.display_name  supervisee_display_name,
      supervisee_email.email  supervisee_email,
      supervisee.department  supervisee_department,
      supervisee.job_title  supervisee_job_title,
      NULL trole_policy_id,
      NULL trole_item_name,
      NULL trole_unique_policy_id,
      NULL trole_description,
      NULL trole_risk,
      NULL trole_policy_violated,
      NULL account_name,
      NULL account_id,
      NULL account_risk,
      NULL account_description,
      NULL account_application_name
    FROM suser_sod_case_info_v users
      LEFT JOIN suser supervisee ON supervisee.supervisor_unique_id = users.unique_user_id AND supervisee.deleted = '0' AND supervisee.effective = '1'
      LEFT JOIN (SELECT DISTINCT 
        suser.id suser,
        suser.display_name,
        STUFF((SELECT DISTINCT ' | ' + suser_email.email
          FROM suser suser1
            JOIN suser_email ON suser.id = suser_email.suser AND suser1.effective = 1 AND suser1.deleted = 0
          WHERE suser1.id = suser.id
          FOR XML PATH('')),1,3,'') email
      FROM suser
        JOIN suser_email ON suser.id = suser_email.suser AND suser.effective = 1 AND suser.deleted = 0
      GROUP BY suser.id, suser.display_name) supervisee_email ON supervisee.id = supervisee_email.suser
    UNION ALL
    SELECT DISTINCT
      users.id,
      users.display_name,
      users.job_title,
      users.email,
      users.department,
      users.user_id,
      users.first_name,
      users.last_name,
      users.risk,
      users.unique_user_id,
      users.supervisor,
      users.violations,
        CASE
          WHEN policy.id IS NOT NULL THEN '5-TROLE'
          ELSE '5-NO-TROLE'
        END detail_type,
      NULL group_id,
      NULL group_name,
      NULL group_description,
      NULL permission_id,
      NULL permission_name,
      NULL permission_description,
      NULL permission_type,
      NULL permission_application_name,
      NULL permission_risk,
      NULL permission_root,
      NULL permission_policy_violated,
      NULL permission_entitlements,
      NULL brole_id,
      NULL brole_name,
      NULL brole_description,
      NULL brole_risk,
      NULL brole_update_time,
      NULL supervisee_id,
      NULL supervisee_display_name,
      NULL supervisee_email,
      NULL supervisee_department,
      NULL supervisee_job_title,
      policy.id  trole_policy_id,
      policy.policy_name  trole_item_name,
      policy.unique_policy_id  trole_unique_policy_id,
      policy.policy_description  trole_description,
      policy.policy_risk  trole_risk,
      sod.policy_name  trole_policy_violated,
      NULL account_name,
      NULL account_id,
      NULL account_risk,
      NULL account_description,
      NULL account_application_name
    FROM suser_sod_case_info_v users
      LEFT JOIN role_perm_policy_v policy ON users.unique_user_id = policy.permission_user_id
      LEFT JOIN sod_case_violation_v sod ON sod.unique_id = policy.unique_policy_id AND users.id = sod.id
    UNION ALL
    SELECT DISTINCT
      users.id,
      users.display_name,
      users.job_title,
      users.email,
      users.department,
      users.user_id,
      users.first_name,
      users.last_name,
      users.risk,
      users.unique_user_id,
      users.supervisor,
      users.violations,
        CASE
          WHEN sac.account_id IS NOT NULL THEN '6-ACCOUNT'
          ELSE '6-NO-ACCOUNT'
        END detail_type,
      NULL group_id,
      NULL group_name,
      NULL group_description,
      NULL permission_id,
      NULL permission_name,
      NULL permission_description,
      NULL permission_type,
      NULL permission_application_name,
      NULL permission_risk,
      NULL permission_root,
      NULL permission_policy_violated,
      NULL permission_entitlements,
      NULL brole_id,
      NULL brole_name,
      NULL brole_description,
      NULL brole_risk,
      NULL brole_update_time,
      NULL supervisee_id,
      NULL supervisee_display_name,
      NULL supervisee_email,
      NULL supervisee_department,
      NULL supervisee_job_title,
      NULL trole_policy_id,
      NULL trole_item_name,
      NULL trole_unique_policy_id,
      NULL trole_description,
      NULL trole_risk,
      NULL trole_policy_violated,
      sac.name  account_name,
      sac.account_id,
      sac.risk  account_risk,
      sac.description  account_description,
      source.name  account_application_name
    FROM suser_account_permission sap
      JOIN saccount sac ON sap.saccount = sac.id AND sac.effective = '1' AND sac.deleted = '0' AND sac.virtual = '0'
      JOIN data_source source ON sac.application = source.id
      RIGHT JOIN suser_sod_case_info_v users ON sap.suser = users.id
  )
  SELECT 
    current_user_access.id,
    current_user_access.display_name,
    current_user_access.job_title,
    current_user_access.email,
    current_user_access.department,
    current_user_access.user_id  user_cn,
    current_user_access.first_name,
    current_user_access.last_name,
    current_user_access.risk,
    current_user_access.unique_user_id,
    current_user_access.supervisor,
    current_user_access.violations,
    current_user_access.detail_type,
    current_user_access.group_id,
    current_user_access.group_name,
    current_user_access.group_description,
    current_user_access.permission_id,
    current_user_access.permission_name,
    current_user_access.permission_description,
    current_user_access.permission_type,
    current_user_access.permission_application_name,
    current_user_access.permission_risk,
    current_user_access.permission_root,
    current_user_access.permission_policy_violated,
    current_user_access.permission_entitlements,
    current_user_access.brole_id,
    current_user_access.brole_name,
    current_user_access.brole_description,
    current_user_access.brole_risk,
    current_user_access.brole_update_time,
    current_user_access.supervisee_id,
    current_user_access.supervisee_display_name,
    current_user_access.supervisee_email,
    current_user_access.supervisee_department,
    current_user_access.supervisee_job_title,
    current_user_access.trole_policy_id,
    current_user_access.trole_item_name,
    current_user_access.trole_unique_policy_id,
    current_user_access.trole_description,
    current_user_access.trole_risk,
    current_user_access.trole_policy_violated,
    current_user_access.account_name,
    current_user_access.account_id,
    current_user_access.account_risk,
    current_user_access.account_description,
    current_user_access.account_application_name
  FROM current_user_access
GO

GRANT SELECT ON business_role_v TO ig_report_role
GRANT SELECT ON suser_account_permission_v TO ig_report_role
GRANT SELECT ON sod_case_info_v TO ig_report_role
GRANT SELECT ON sod_policy_v TO ig_report_role
GRANT SELECT ON sod_policy_all_v TO ig_report_role
GRANT SELECT ON policy_owners_v TO ig_report_role
GRANT SELECT ON business_role_nested_v TO ig_report_role
GRANT SELECT ON sgroups_subgroups_v TO ig_report_role
GRANT SELECT ON sgroup_path_v TO ig_report_role
GRANT SELECT ON sgroup_indirect_suser_v TO ig_report_role
GRANT SELECT ON suser_sod_case_info_v TO ig_report_role
GRANT SELECT ON spermission_suser_v TO ig_report_role
GRANT SELECT ON spermission_associations_v TO ig_report_role
GRANT SELECT ON sod_policy_violation_v TO ig_report_role
GRANT SELECT ON role_perm_policy_v TO ig_report_role
GRANT SELECT ON sod_case_violation_v TO ig_report_role
GRANT SELECT ON deleted_reviewers_v TO ig_report_role
GRANT SELECT ON suser_brole_v TO ig_report_role
GRANT SELECT ON current_user_access_v TO ig_report_role
GO